#include <stdio.h>
#include <errno.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <dlfcn.h>

static void *load_sym(const char *name)
{
	void *addr;
	const char *msg;

	(void) dlerror();
	addr = dlsym(RTLD_NEXT, name);
	if ((msg = dlerror()))
	{
		fprintf(stderr, "dlsym(%s): %s\n", name, msg);
		_exit(1);
	}
	return addr;
}

int link(const char *oldpath, const char *newpath)
{
	errno = ENOSYS;
	return -1;
}

int mkdir(const char *pathname, mode_t mode)
{
	errno = ENOSYS;
	return -1;
}

int symlink(const char *oldpath, const char *newpath)
{
	errno = ENOSYS;
	return -1;
}

int chmod(const char *path, mode_t mode)
{
	static int (*next_chmod)(const char *, mode_t);

	if (!next_chmod)
		next_chmod = load_sym("chmod");

	return next_chmod(path, 0644);
}

int fchmod(int fd, mode_t mode)
{
	static int (*next_fchmod)(int, mode_t);

	if (!next_fchmod)
		next_fchmod = load_sym("fchmod");

	return next_fchmod(fd, 0644);
}
